import os  # os.split
import sys  # sys.stdout
import telnetlib  # telnet connection to gns3 routers
import time  # time.sleep
import argparse
import json  # read .gns3
from multiprocessing.dummy import Pool as ThreadPool  # threads used to get confs
import threading

# GLOBAL VARS #############################

# Empty list used to return gns3 routers output
confs_routeurs = []


#Filter => quelles infos virer de la conf'
mpls_filter = ["root>","show configuration protocols mpls","interface all", "interface ge-0/0/1.0", "interface ge-0/0/2.0", "interface ge-0/0/3.0", "interface ge-0/0/4.0","[edit]",";","interface em1.0","interface em2.0","interface em3.0","interface em4.0"]
bgp_filter = []

router_filter = ["CE1", "CE2", "CE3", "RRF"]

# router name <=> port correspondance
dict_name_port = {}

# Server running gns3
routeurs_addr = "127.0.0.1"

# GNS3 Project paths
confs_file_path = "Juniper_Topology2"
gns_file = "Juniper_Topology2/Juniper_Topology2.gns3"

# Options
verbose = False
latex = False
threadlock = threading.Lock()

# END GLOBAL VAR ###########################


# TELNET FUNCTION

# Clean telnet console
def flush(tn):
    tn.write("\r\n\r\ncli\r\n\r\n\r\n")
    time.sleep(10)
    tn.read_very_eager()


def filter_info(info, filter):
    for x in filter:
        info = info.replace("{0}".format(x), "")
    info_lines = info.split("\n")
    new_contents = []
    for i in info_lines:
        if not i.strip() or i in filter:
            continue
        else:
            new_contents.append("{0}\n".format(i))

    return "".join(new_contents)

def get_section(name, tn, section):
    initiated_tn = False
    if tn is None:
        tn = telnetlib.Telnet(routeurs_addr, dict_name_port[name])
        initiated_tn = True
    flush(tn)
    tn.write("\r\n\r\nshow configuration {0}\r\n".format(section))
    time.sleep(10)
    out = tn.read_very_eager()
    processed_out = ""
    for line in out.split("\r"):
        if "#" not in line:
            processed_out = processed_out + line
    return processed_out  # processed_out

    if initiated_tn:
        tn.close()


def get_running_conf(name):
    if name in router_filter:
        return
    if verbose:
        sys.stdout.write("In thread\n")
    tn = telnetlib.Telnet(routeurs_addr, dict_name_port[name])
    flush(tn)


    # get mpls info
    mpls = get_section(name, tn, "protocols mpls")
    mpls = filter_info(mpls, mpls_filter)
    if(len(mpls) == 0):
        mpls="propagate ttl"

    conf_routeur = "%s\n\n%s\n" % (name, mpls)

    with threadlock:
        confs_routeurs.append(conf_routeur)
    tn.close()
    return conf_routeur


# set router ldp config to explicit null
def set_uhp(name):
    if verbose:
        sys.stdout.write("Setting UHP on {0}\n".format(name))
    tn = telnetlib.Telnet(routeurs_addr, dict_name_port[name])
    flush(tn)
    tn.write("\r\n configure \r\n")
    tn.write("\r\n set protocols mpls explicit-null \r\n")
    tn.write("\r\n commit \r\n")
    time.sleep(2)
    tn.write("\r\n exit \r\n")
    time.sleep(10)
    tn.close()


# set router ldp config to implicit null
def set_php(name):
    if verbose:
        sys.stdout.write(
            "Setting PHP on {0}:{1}\n".format(name,
                                              dict_name_port[name]))
    tn = telnetlib.Telnet(routeurs_addr, dict_name_port[name])
    flush(tn)
    tn.write("\r\n configure \r\n")
    tn.write("\r\n delete protocols mpls explicit-null \r\n")
    tn.write("\r\n commit \r\n")
    time.sleep(2)
    tn.write("\r\n exit \r\n")
    time.sleep(10)
    tn.close()


# set ip propagate on router
def set_prop(name):
    if verbose:
        sys.stdout.write(
            "Setting ip propagate on {0}:{1}\n".format(name,
                                                       dict_name_port[name]))
    tn = telnetlib.Telnet(routeurs_addr, dict_name_port[name])
    flush(tn)
    tn.write("\r\n configure \r\n")
    tn.write("\r\n delete protocols mpls no-propagate-ttl \r\n")
    tn.write("\r\n commit \r\n")
    time.sleep(2)
    tn.write("\r\n exit \r\n")
    time.sleep(10)
    tn.close()

# set icmp tunneling
def set_icmp_tun(name):
    if verbose:
        sys.stdout.write(
            "Setting ip propagate on {0}:{1}\n".format(name,
                                                       dict_name_port[name]))
    tn = telnetlib.Telnet(routeurs_addr, dict_name_port[name])
    flush(tn)
    tn.write("\r\n configure \r\n")
    tn.write("\r\n set protocols mpls icmp-tunneling \r\n")
    tn.write("\r\n commit \r\n")
    time.sleep(2)
    tn.write("\r\n exit \r\n")
    time.sleep(10)
    tn.close()


# set ip no propagate ttl on router
def set_noprop(name):
    if verbose:
        sys.stdout.write("Setting no ip propagate on {0}\n".format(name))
    tn = telnetlib.Telnet(routeurs_addr, dict_name_port[name])
    flush(tn)
    tn.write("\r\n configure \r\n")
    tn.write("\r\n set protocols mpls no-propagate-ttl \r\n")
    tn.write("\r\n commit \r\n")
    time.sleep(2)
    tn.write("\r\n exit \r\n")
    time.sleep(10)
    tn.close()


# END TELNET

# print router configuration

def print_confs(confs):
    confs = sorted(confs)
    second_half = False
    if latex:
        sys.stdout.write("\\begin{minipage}{0.5\\textwidth}" + '\n')
        sys.stdout.write("\\begin{verbatim}")
        for i in xrange(0, len(confs)):
            if i < len(confs) / 2 +1 :
                sys.stdout.write(confs[i])
                sys.stdout.write("--\n\n")
            else:
                if not second_half:
                    sys.stdout.write("\\end{verbatim}")
                    sys.stdout.write("\\end{minipage}% \n \\hfill")
                    sys.stdout.write("\\begin{minipage}{0.5\\textwidth}")
                    sys.stdout.write("\\begin{tabular}{|p{\\textwidth}}\n")
                    sys.stdout.write("\\begin{verbatim}")
                    second_half = True
                sys.stdout.write(confs[i])
                sys.stdout.write("----\n\n")
        sys.stdout.write("\\end{verbatim}")
        sys.stdout.write("\\end{tabular}")
        sys.stdout.write("\\end{minipage}%\n")
        return

    for conf in confs:
        sys.stdout.write("\n=========================\n")
        sys.stdout.write(conf)
        sys.stdout.write("\n=========================\n")




def name_port():
    if verbose:
        sys.stdout.write("Console Port  <-> Name ( building dictionnary )\n")
    with open(gns_file) as gns_fd:
        gns_json = json.load(gns_fd)
        for nodes in gns_json['topology']['nodes']:
            if nodes['console'] is not None and nodes['console'] != "null":
                dict_name_port[nodes['name']] = int(nodes['console'])
                if verbose:
                    sys.stdout.write(
                        "{0} {1}\n".format(nodes['console'], nodes['name']))

# END MISC


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("-path", type=str)
    parser.add_argument("-verbose", action='store_true')

    parser.add_argument("-uhp", nargs='*')
    parser.add_argument("-php", nargs='*')
    parser.add_argument("-no_prop", nargs='*')
    parser.add_argument("-prop", nargs='*')
    parser.add_argument("-icmp_tun", nargs='*')

    args = parser.parse_args()

    if args.path is not None:
        gns_file = args.path
        head, _ = os.path.split(gns_file)
        confs_file_path = head

    if args.verbose is True:
        verbose = True
    name_port()

        # Launch threads
    number_of_threads = len(dict_name_port.keys())
    pool = ThreadPool(number_of_threads)



    if args.uhp is not None:
        pool_args = []
        for router in args.uhp:
            pool_args.append(router)
        res = pool.map(set_uhp, pool_args)
        time.sleep(5)

    if args.php is not None:
        pool_args = []
        for router in args.php:
            pool_args.append(router)
        res = pool.map(set_php, pool_args)
        time.sleep(5)

    if args.no_prop is not None:
        pool_args = []
        for router in args.no_prop:
            pool_args.append(router)
        res = pool.map(set_noprop, pool_args)
        time.sleep(5)

    if args.prop is not None:
        pool_args = []
        for router in args.prop:
            pool_args.append(router)
        res = pool.map(set_prop, pool_args)
        time.sleep(5)

    if args.icmp_tun is not None:
        pool_args = []
        for router in args.icmp_tun:
            pool_args.append(router)
        res = pool.map(set_icmp_tun, pool_args)
        time.sleep(5)


    
